/*  symtab.h - XML Symbol Table Module
    Copyright (c) 2008 Robert D. Cameron
    Licensed to the public under the Open Software License 3.0.
    Licensed to International Characters, Inc., under the Academic
    Free License 3.0.
*/

#ifndef SYMTAB_H
#define SYMTAB_H

#include "namechars.h"
#include "stringpool.h"

struct eqstr
{
  bool operator()(const char* s1, const char* s2) const
  {
    return strcmp(s1, s2) == 0;
  }
};

struct Name_Data
{
	char * name_string;
	int	   lgth;
};

const char * predefined[] = {"lt", "gt", "amp", "quot", "apos"};

class Symbol_Table {
public:
	Symbol_Table();


	int Insert_Name(const char * name, int lgth);
	int ASCII_Lookup_or_Insert_Name(char * name, int lgth);
	int UTF8_Lookup_or_Insert_Name(char * name, int lgth);
	int ASCII_Lookup_or_Insert_Nmtoken(char * name, int lgth);
	int UTF8_Lookup_or_Insert_Nmtoken(char * name, int lgth);

	char * Get_UTF8_name(int nameID);
	int Get_UTF8_lgth(int nameID);

	char * Get_UTF8_nmtoken(int nmtokenID);
	int Get_UTF8_nmtoken_lgth(int nmtokenID);

	char * ReserveSymbolSpace(int u8_lgth);
	int LookupOrInsertReserved();
	int LookupOrInsertReserved_nmtoken();

	XML_version version;
	vector<Name_Data> UTF8NameTable;
	vector<Name_Data> UTF8NmtokenTable;
private:
	hash_map<const char *, int, hash<const char *>, eqstr > UTF8NameMap;
	hash_map<const char *, int, hash<const char *>, eqstr > UTF8NmtokenMap;
	int globalNameCount;
	int globalNmtokenCount;
//	vector<char *> UTF8NameTable;

	char * reserved;
	int reserved_lgth;
	StringPool<4096,100> pool;
};



#endif
