using System;
using System.IO;
using System.Xml;

namespace XMLBench {
 abstract public class XMLBench {
    public string xslt_file;
    
    public void Bench(string[] args) {
	if (args.Length<2) {
	    Console.WriteLine("Usage:\n\tme <iterations> <xml file|xmlgen|opcgen|xmark> [size|xsl_file]");
	}

	int iterations = Convert.ToInt32(args[0]);
	
	bool mode;
	if ((args[1] == "xmlgen")||(args[1] == "opcgen")||(args[1] == "xmark")) {
	    switch (args[1]) {
	     case "xmlgen":
	        xslt_file = "../xml.files/generated.xsl";
	     break;
	     case "opcgen":
	        xslt_file = "../xml.files/opc.xsl";
	     break;
	     default:
	        xslt_file = args[2];
	     break;
	    }
	    mode = true;
	} else {
	    mode = false;
	    xslt_file = args[2];
	}
	
	DateTime pre, post;
	TimeSpan ts = new TimeSpan(0);
	Disp d = new Disp();
	
	InitXML();
	
	FileStream xml;
	
	if (mode) xml = new FileStream("../xml.tmp/0.xml", FileMode.Open);
	else xml = new FileStream(args[1], FileMode.Open);
	
	ParseXML(xml, iterations, 0);
	
	xml.Close();
	
	for (int i = 0; i < iterations; i++) {
	    if (mode) xml = new FileStream("../xml.tmp/" + (i+1) + ".xml", FileMode.Open);
	    else xml = new FileStream(args[1], FileMode.Open);
	    
    	    pre = DateTime.Now;
	    ParseXML(xml, iterations, 0);
	    post = DateTime.Now;
	    d.disp_event((post - pre).TotalMilliseconds);
	    
	    xml.Close();
	    
	    ts += post - pre;

	}
	d.disp_post();

	TimeSpan vts = new TimeSpan(0);
	Disp dv = new Disp();

	InitXML_Transformation(xslt_file);

	if (mode) xml = new FileStream("../xml.tmp/0.xml", FileMode.Open);
	else xml = new FileStream(args[1], FileMode.Open);
	
	TransformXML(xml, iterations, 0);
	
	xml.Close();
	
	for (int i = 0; i < iterations; i++) {
	    if (mode) xml = new FileStream("../xml.tmp/" + (i+1) + ".xml", FileMode.Open);
	    else xml = new FileStream(args[1], FileMode.Open);
	    
    	    pre = DateTime.Now;
	    TransformXML(xml, iterations, 0);
	    post = DateTime.Now;
	    dv.disp_event((post - pre).TotalMilliseconds);
//	    Console.WriteLine((post - pre).TotalMilliseconds);
	    
	    xml.Close();
	    
	    vts += post - pre;

	}
	dv.disp_post();
	
	ReleaseXML();
	
	ts = vts - ts;

	if (args[1] == "opcgen") {
	    Console.WriteLine("Mono Transformation Time: " + ts.TotalMilliseconds + " ms");
	} else {
//	    Console.WriteLine("Mono Time: " + ts.TotalMilliseconds/iterations + " ms");
//	    Console.WriteLine("Mono Time: " + (dv.disp_m-d.disp_m) + "(" + (300*(d.disp_d+dv.disp_d)/(dv.disp_m-d.disp_m)) + "%)");
	    Console.WriteLine("Mono Transformation Time: " + (ts.TotalMilliseconds/iterations) + "(" + (300*(d.disp_d+dv.disp_d)/(dv.disp_m-d.disp_m)) + "%)");
	}
    }
    virtual public void InitXML() {}
    virtual public void InitXML_Transformation(string xslt_file) {}
    virtual public void ReleaseXML() {}
    abstract public void ParseXML(Stream xml, int iters, int iter);
    abstract public void TransformXML(Stream xml, int iters, int iter);
 }
}
